
# q-expansion of newform 1520.2.bq.p, downloaded from the LMFDB on 05 February 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1520
weight = 2
poly_data = [13, 28, 18, 16, 8, -4, 4, -3, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0], 1], [[41, 40, 22, 6, -2, 2, -2, 1], 27], [[-86, -100, -13, -99, 53, -44, 26, -7], 27], [[-58, -48, -14, -50, 29, -20, 11, -3], 9], [[-169, -107, -56, -150, 97, -79, 43, -11], 27], [[257, 178, 58, 201, -131, 95, -50, 13], 27], [[610, 437, 227, 477, -292, 229, -130, 35], 27]]

hecke_ring_character_values = [[191, [-1, 0, 0, 0, 0, 0, 0, 0]], [1141, [1, 0, 0, 0, 0, 0, 0, 0]], [1217, [1, 0, 0, 0, 0, 0, 0, 0]], [401, [0, 0, 0, 0, 0, 0, 1, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [0, 0, -1, 0, 0, -1, 0, 0], [-1, 0, 0, 0, 0, 0, 1, 0], [-1, 1, 1, -2, -1, 1, 0, 2], [-2, 1, 1, -2, -1, 1, 2, 2], [-3, 1, -1, 1, 0, -1, 1, 0], [0, -1, 1, 0, 1, 2, 0, 0], [1, -1, 0, 0, -2, 1, -2, -1], [3, -2, 2, -1, 0, 2, -1, 0], [-2, 1, -2, 0, 1, -2, 1, 0], [6, -2, 2, 0, -1, 1, 0, 0], [1, -1, -1, 2, 3, 1, 0, -2], [0, -3, 0, 0, -3, 0, 0, -3], [-1, -3, 0, 0, -3, 3, -1, 0], [4, -1, 0, 2, 1, 0, -3, 0], [6, -3, -1, 2, 4, -1, -4, 0], [4, -1, 2, -2, 1, 4, -4, 1], [-1, 1, -1, -1, 2, 3, 5, 2], [-3, 5, 1, -3, 4, 5, 6, 6], [-9, 5, 0, 0, -5, -5, 9, 0], [5, -1, 1, -4, 1, 4, -5, 2], [-2, 2, -2, 4, -2, -6, 2, -2], [-2, -2, -2, -3, -2, 1, 1, 3], [8, -2, -2, 0, 4, -2, -4, 0], [0, -4, 3, 0, -4, 5, 0, 4], [0, -5, -1, 0, -4, 1, 4, 0], [7, 0, 0, 1, 4, -3, -1, 1], [-7, -2, 2, -1, -6, 5, 1, -1], [5, -3, 0, 0, -3, -3, 5, -6], [6, -5, -5, 6, 1, -5, -6, -6], [0, -4, 2, 0, -6, -2, -6, 0], [2, 3, -4, 0, 3, -4, 2, -5], [-1, -3, 0, -1, -3, 2, 1, 2], [-2, -6, 6, 0, 0, 6, 1, 0], [-4, 4, 8, -6, -4, 7, 4, 3], [1, 2, -2, 1, 3, -2, -1, 1], [-1, 2, 1, 2, -2, -2, 1, -1], [0, 2, 2, -3, 2, 5, -3, 3], [0, 5, 2, 0, 3, -2, 3, 0], [-7, 2, 3, 0, 2, -1, -7, 4], [-3, 4, -4, 0, 1, -1, 0, 0], [3, -1, 7, -1, -6, 7, -1, 0], [6, -2, -2, 0, -2, -2, -12, 0], [4, 3, 3, 0, 3, -6, 4, 0], [-5, 7, -7, 0, 5, -5, 0, 0], [8, -2, -1, 2, 3, -1, -5, 0], [4, 6, 6, -8, -6, 4, -4, 4], [5, -3, 4, -10, 3, 12, -5, 5], [-6, -6, 6, -3, -11, 8, 3, -3], [14, -3, 3, 1, 0, 1, -1, 1], [6, 1, -10, 4, -1, -13, -6, -2], [0, 0, 0, -4, -1, 3, -4, 4], [-7, 9, -5, -5, -4, -5, 6, 0], [2, -1, 0, -10, 1, 0, 4, 0], [2, 3, -6, 6, 3, -6, -4, 0], [2, -2, 0, 0, -2, -6, 2, -8], [0, 13, -9, 0, 13, -5, 0, -1], [4, 4, 4, 0, 4, -5, 4, 3], [-4, -6, 6, 2, -6, 8, -2, 2], [0, -4, 3, 4, 1, 3, -2, 0], [-6, 1, -2, 0, 1, -5, -6, -6], [11, -2, -2, 3, -2, -5, -19, -3], [0, 6, 3, -12, -6, 3, 0, 6], [-5, 5, 5, -8, -9, -1, 2, 8], [-2, 11, 7, -2, 4, -3, -1, 4], [4, -4, -1, 6, 5, -1, -5, 0], [12, -1, 1, 0, 1, -1, 0, 0], [7, 5, -1, 0, 5, -1, 7, 3], [12, -2, -5, 0, -2, -2, 12, -9], [6, -11, 11, 3, -7, 10, -3, 3], [8, -12, 12, -3, -6, 3, 3, -3], [0, -9, 7, 0, -9, -2, 0, -4], [-13, 3, -11, -1, 8, -11, 7, 0], [9, -5, -5, 0, -2, -2, -18, 0], [-13, 5, -5, -4, 4, -8, 4, -4], [-1, -5, -2, -4, 5, 5, 1, 2], [3, -2, 5, 3, -7, -11, 8, -6], [-7, 10, 11, -8, -10, 5, 7, 4], [6, -6, -1, 0, -6, 0, 6, -7], [2, 11, -15, 4, 4, -15, -3, 0], [4, 2, 2, 6, -1, -7, -2, -6], [-5, 11, -2, 0, 11, 8, -5, 17], [0, 5, 0, 0, 5, 0, 3, 0], [-7, 2, -5, 11, 3, -5, -2, 0], [9, 3, 8, 6, -3, 2, -9, -3], [-13, -10, 7, 7, 3, 7, 3, 0], [5, 2, 2, 2, -3, -5, -8, -2], [0, 0, 0, -7, -6, -1, 7, -7], [7, 0, -9, 8, 0, -13, -7, -4], [13, -14, -14, 9, -2, -11, -17, -9], [3, 2, 2, 10, -6, -16, 4, -10], [-15, 3, 0, 3, -3, 0, 6, 0], [-8, 0, 0, -2, -3, 1, 2, -2], [-14, 6, -6, 0, 6, 15, -14, 15], [2, 2, -1, 0, 2, -4, 2, -3], [-1, 10, -9, 1, -1, -9, 0, 0], [16, 5, -1, -12, -4, -1, -2, 0], [10, -9, -9, 6, 6, 0, -14, -6], [3, 0, -1, 3, 1, -5, 12, -6], [0, 8, -5, 0, 13, 5, -1, 0], [-4, 11, 9, -4, 2, -1, 10, 8], [2, -10, -11, 2, 1, 7, 1, -4], [15, -10, 10, 0, -6, 6, 0, 0], [-2, -8, -8, -6, 4, 10, -2, 6], [1, 4, 4, -6, -2, 4, -8, 6], [10, 2, -2, 5, 10, -5, -5, 5], [-1, 2, -7, 0, 2, 8, -1, 3], [5, 7, -1, 5, 8, -9, -4, -10], [3, 3, -3, 3, 6, -3, 21, -6], [-6, 6, 6, -1, -5, -4, 11, 1], [1, 0, 0, -5, 6, -11, 5, -5], [-2, -4, 10, 2, 4, 13, 2, -1], [4, -13, -11, 4, -2, 3, -15, -8], [0, 1, 1, 3, 4, 1, 3, -3], [-22, -2, 10, -6, -8, 10, 14, 0], [1, -2, 0, 0, -2, 7, 1, 5], [14, 1, 4, 0, 1, -8, 14, -3], [5, 9, 9, -8, -2, 6, -18, 8], [-12, -1, 1, 7, 4, 3, -7, 7], [3, -10, -4, 3, -6, -2, 3, -6], [-8, -5, 8, -2, -3, 8, 5, 0], [6, -7, -7, -1, 0, 1, -13, 1], [-11, -3, -3, 2, -8, -10, 24, -2], [-24, 5, -5, -3, 0, -3, 3, -3], [5, 3, 3, 4, -11, -15, -6, -4], [-11, -9, -6, 22, 9, -8, 11, -11], [0, -1, -2, 0, 1, 2, 19, 0], [0, 7, -3, 0, 7, 3, 0, 7], [5, -5, 4, 0, -5, 1, 5, 0], [0, -3, 3, 7, 0, 7, -7, 7], [-5, 10, 0, 0, 10, -3, -5, 7], [17, 3, 14, 8, -3, 7, -17, -4], [-6, 13, -6, -6, 19, 18, -3, 12], [8, -1, -5, 22, 1, -15, -8, -11], [1, -1, 1, 7, 7, 0, -7, 7], [2, 3, -15, 2, 18, 11, -14, -4], [-9, -5, 0, 5, 5, 0, 2, 0], [20, -10, 10, 5, -8, 13, -5, 5], [-6, -9, -9, -3, 6, 9, 9, 3], [0, 14, -14, -1, 16, -17, 1, -1], [-3, 1, 1, -3, 0, 5, 3, 6], [1, 7, -10, 1, 17, 8, -10, -2], [-18, 6, 0, -6, -6, 0, 12, 0], [-6, 5, 0, -6, 5, 12, 30, 12], [5, 7, 7, -8, 6, 14, -18, 8], [4, -1, -7, 4, 1, -8, -4, -2], [-22, 10, 14, -18, -10, 13, 22, 9], [-16, 3, 0, 0, 3, 18, -16, 21], [-11, -7, -4, 21, 11, -4, -5, 0], [-1, 9, -9, -4, -6, 2, 4, -4], [-7, 4, -7, 0, 4, 7, -7, 4], [-10, 4, -4, -7, -1, -6, 7, -7], [3, -1, -8, 19, 9, -8, -11, 0], [3, -1, -4, 3, 3, -2, -12, -6], [9, 7, -7, 6, -7, -17, -9, -3], [-29, 2, -2, -5, 3, -8, 5, -5], [-22, 11, 11, -1, -11, -10, 43, 1], [12, 6, -15, 10, -6, -26, -12, -5], [2, 10, 5, 2, 5, -9, 6, -4], [-2, 14, -10, -6, -4, -10, 4, 0], [-1, -2, 1, 0, -2, 10, -1, 9], [-8, 12, -7, 0, 12, 9, -8, 14], [-4, -4, 12, 0, -4, 6, -4, 14], [2, 1, -6, 8, -1, -11, -2, -4], [-3, -8, -8, 8, -1, -9, 14, -8], [-11, 5, 6, -20, -5, 11, 11, 10], [12, -16, -8, 24, 16, -4, -12, -12], [5, 1, 5, 5, -4, -15, 21, -10], [24, -10, 10, 0, -10, -10, 24, -10], [4, 4, -1, 4, 5, -7, 33, -8], [-7, 3, -3, 11, 15, -4, -11, 11], [15, -5, 12, 0, -5, -11, 15, -4], [-4, 25, -19, 0, 25, -1, -4, 5], [-1, 3, -3, -5, 0, -5, 5, -5], [-2, -15, -3, -2, -12, 7, 14, 4], [-4, -13, 7, -4, -20, 1, 1, 8], [5, -7, -9, 11, 16, -9, -8, 0], [20, -12, -24, 14, 12, -19, -20, -7], [-5, -17, 17, 4, -10, 14, -4, 4], [-9, 4, -1, 8, -4, -9, 9, -4], [-18, -13, -3, 14, 16, -3, 2, 0], [15, -1, 1, -6, -6, 0, 6, -6], [-26, 10, 8, -2, -18, 8, 14, 0], [6, -8, 7, 0, -8, -11, 6, -12], [-4, -4, -4, 12, -4, -16, 20, -12], [1, 6, -6, 13, 12, 1, -13, 13], [-4, 0, 11, -10, -11, 11, 7, 0], [-3, 12, 17, -12, -12, 11, 3, 6], [18, 6, 6, -1, 4, 5, -37, 1], [2, -3, -3, 7, -5, -12, 3, -7], [26, -3, 16, -2, -13, 16, -12, 0], [26, -3, -14, 8, 17, -14, -17, 0], [-4, 11, -11, 2, 5, -3, -2, 2], [22, -2, 20, 0, -2, -20, 22, -2], [43, -1, 0, 11, 1, 0, -27, 0], [-17, 5, 7, -13, -12, 7, 15, 0], [-10, 12, 18, -10, -6, 2, 9, 20], [-2, 2, -20, 0, -2, -22, 2, 0], [-8, 4, -4, -5, 5, -10, 5, -5], [20, -5, -5, -3, 7, 10, -43, 3], [26, 1, -17, 18, 16, -17, -22, 0], [12, -17, -7, 12, -10, -17, -12, -24], [-2, 22, -10, 0, 22, -14, -2, -2], [12, 2, -5, 0, 2, -4, 12, -7], [-9, -9, 11, 0, -9, -1, -9, 1], [2, -20, -4, 2, -16, 0, 0, -4], [-24, 5, -17, 4, 12, -17, 10, 0], [-7, 2, 11, -7, -9, 3, 1, 14], [-14, 12, -3, -2, -12, -14, 14, 1], [-29, -18, 18, -2, -17, 15, 2, -2], [5, 1, -1, 20, -1, -12, -5, -10], [-8, 22, 17, -8, 5, -1, -7, 16], [2, 6, 6, -6, 9, 15, -10, 6], [-1, -14, -8, -1, -6, 10, -26, 2], [6, 18, -18, 6, 13, -7, -6, 6], [34, -3, 3, 6, 0, 3, -20, 0], [3, -15, 12, 0, -15, -3, 3, -6], [8, 1, 1, -11, -6, 5, -27, 11], [-21, 20, -20, -3, 11, -14, 3, -3], [-21, 16, 23, -16, -16, 15, 21, 8], [24, -23, 20, -2, 3, 20, -11, 0], [40, -11, 11, 1, -14, 15, -1, 1], [2, 9, -4, 0, 9, -18, 2, -13], [4, -23, 23, 0, -23, 2, 4, 2], [10, -5, -2, -20, 5, 13, -10, 10], [-10, -9, 9, -10, -18, 11, 12, 20], [27, 0, 0, 8, -5, -13, -46, -8], [-35, -7, 12, -2, 7, 20, 35, 1], [13, -20, -3, 22, 20, 6, -13, -11], [0, 2, 2, -6, 17, 23, -6, 6], [-1, -12, 0, -1, -12, 2, 17, 2], [-1, 23, 10, -1, 13, -8, -23, 2], [21, -5, 5, -6, -4, -2, 6, -6], [-14, -1, -1, 2, -15, -17, 30, -2], [-11, 9, 9, -9, -27, -18, 13, 9], [11, 8, 3, 0, 8, 0, 11, 11], [15, -25, 25, -5, -20, 15, 5, -5], [15, -20, 20, -3, -25, 22, 3, -3], [-23, 2, 2, 0, 2, -10, -23, -6], [-4, 11, -1, -4, 12, 9, 4, 8], [-5, 8, -2, -2, -8, -9, 5, 1], [16, -3, -3, 0, 9, 9, -32, 0], [-30, 21, -24, 0, 3, -24, 15, 0], [19, -10, -3, 1, 13, -3, -10, 0], [29, 3, -2, 0, 3, 3, 29, 4], [-2, 3, 3, -7, -16, -9, -3, 7], [4, -7, -7, 12, 14, -7, -8, 0], [1, 5, 3, 1, 2, -5, 14, -2], [38, -8, -21, 20, 8, -23, -38, -10], [-9, -3, 16, -19, -13, 16, 14, 0], [4, 2, -2, 2, 16, -14, -2, 2], [29, 3, 18, -4, -3, 17, -29, 2], [-8, 21, 21, -23, -14, 9, -7, 23], [3, 21, -20, -5, -1, -20, 1, 0], [39, -15, 0, -4, 15, 17, -39, 2], [35, -22, 16, -5, 6, 16, -15, 0], [0, -9, -3, 0, -6, 3, 0, 0], [17, -11, 11, -1, 8, -9, 1, -1], [-10, 2, -10, 0, 2, 17, -10, 9], [-16, 16, 17, -16, -1, 15, -13, 32], [-6, 18, -6, -6, 24, 18, 12, 12], [-17, 7, -19, 4, -7, -28, 17, -2], [16, -18, -12, 10, 18, 1, -16, -5], [-3, 31, -22, 0, 31, -11, -3, -2], [-2, 8, -5, -2, 13, 9, -2, 4], [17, 4, -4, 2, 3, -1, -2, 2], [-6, 19, 19, -1, -13, -12, 11, 1], [-23, 12, -12, -6, 9, -15, 6, -6], [-5, 2, -4, -5, 6, 14, -49, 10], [0, 16, -13, 0, 29, 13, 9, 0], [2, -4, -4, 4, -20, -24, 0, -4], [6, 7, -7, 9, 9, 0, -9, 9], [-39, -5, -1, 8, 5, 0, 39, -4], [-19, -4, -4, -3, -10, -7, 35, 3], [56, -3, -4, 14, 7, -4, -35, 0], [15, 8, -1, 0, 8, 2, 15, 9], [-3, 7, 7, 19, -13, -32, 25, -19], [0, 11, -9, 0, 11, -22, 0, -20], [-41, -4, -11, 11, 15, -11, 15, 0], [11, -25, 18, 5, 7, 18, -8, 0], [-1, 10, -10, -1, 4, -5, 1, -1], [-12, 0, 0, -6, 15, 21, 18, 6], [15, -9, 16, -7, -7, 16, -4, 0], [-3, -1, -1, -2, 13, 15, 4, 2], [17, -4, -15, 0, -4, 6, 17, -13], [28, -9, 20, 10, -11, 20, -19, 0], [-8, -16, 16, -9, -26, 17, 9, -9], [22, -8, 1, 24, 7, 1, -23, 0], [-3, -10, -9, 0, -10, 0, -3, -19], [-16, 12, 12, 2, -5, -7, 34, -2], [0, -3, 3, -3, 6, -9, 3, -3], [-24, 13, 13, -21, -14, 7, 27, 21], [-19, 5, 19, -1, -24, 19, 10, 0], [8, 15, -15, -13, -4, -9, 13, -13], [-9, -9, -6, 0, -9, 0, -9, -15], [26, 6, -6, -12, -9, -3, 12, -12], [2, -3, -6, 2, 3, 2, -35, -4], [-23, 16, -19, -6, -16, -32, 23, 3], [21, -23, -28, 6, 23, -8, -21, -3], [11, 3, 3, 3, -15, -18, -19, -3], [-26, -2, 11, -22, 2, 24, 26, 11], [-10, -19, 5, 0, -19, 2, -10, -12], [-42, -3, 26, -20, -23, 26, 31, 0]]
